from ..main_exceptions import UnauthorizedException


class IncorrectTokenFormatException(UnauthorizedException):
    def __init__(self):
        super().__init__(code="TOKEN_INCORRECT", message="Неверный формат токена.")


class UserUnauthorizedException(UnauthorizedException):
    def __init__(self):
        super().__init__(message="Пользователь не авторизован.")


class TokenExpiredException(UnauthorizedException):
    def __init__(self):
        super().__init__(code="TOKEN_EXPIRED", message="Токен истек.")



from fastapi import status

from fastapi.exceptions import HTTPException
from fastapi.encoders import jsonable_encoder

from ..schemas.schemas import ResponseDetail


class ServiceException(HTTPException):
    def __init__(self, status_code: int = 500, code: str = None, message: str = None):
        self.status_code = status_code
        self._detail = ResponseDetail(code=code, message=message)
        super().__init__(status_code=self.status_code, detail=jsonable_encoder(self._detail))


class UnauthorizedException(ServiceException):
    def __init__(self, code="UNAUTHORIZED", message="Клиент не идентифицирован."):
        super().__init__(status_code=status.HTTP_401_UNAUTHORIZED, code=code, message=message)


class ForbiddenException(ServiceException):
    def __init__(self, code="FORBIDDEN", message="Доступ запрещен."):
        super().__init__(status_code=status.HTTP_403_FORBIDDEN, code=code, message=message)



import jwt

from jwt.exceptions import ExpiredSignatureError, InvalidTokenError
from starlette.requests import Request

from .exceptions.sub_exceptions.unauthorized_exceptions import TokenExpiredException, IncorrectTokenFormatException, \
    UserUnauthorizedException


def get_token_from_request(request: Request) -> str:
    """
    Extract and return token from request headers.

    :param request: an http request
    :type request: Request
    :returns: token
    :rtype: str
    """
    if not request.headers.get('Authorization'):
        raise UserUnauthorizedException
    try:
        token = request.headers.get('Authorization').split(" ")[1]
        return token
    except Exception as e:
        # Если IndexError, скорее всего токен передан без префикса "Bearer "
        print(f"Ошибка в методе get_token.\n"
              f"Тип исключения: {type(e).__name__}\nСообщение: {str(e)}\n"
              f"Переданный токен: {request.headers.get('Authorization')}")
        raise IncorrectTokenFormatException


def decode_token(token: str, public_key: bytes, jwt_algorithm: str) -> dict:
    """
    Decode jwt token and return payload dictionary.

    :param token: jwt token to decode
    :type token: str
    :param public_key: public key used to decode the token
    :type public_key: bytes
    :param jwt_algorithm: jwt algorithm used to decode the token
    :type jwt_algorithm: str
    :returns: payload from the token
    :rtype: dict
    """
    try:
        payload = jwt.decode(
            token, public_key, jwt_algorithm
        )
        return payload
    except ExpiredSignatureError:
        raise TokenExpiredException
    except InvalidTokenError as e:
        print(e)
        raise IncorrectTokenFormatException


def validate_token(token: str, public_key: bytes, jwt_algorithm: str):
    """
    Validate jwt token. Nothing is returned in case of successful validation. An exception is thrown otherwise.

    :param token: jwt token
    :type token: str
    :param public_key: public key
    :type public_key: bytes
    :param jwt_algorithm: jwt algorithm
    :type jwt_algorithm: str
    """
    decode_token(token=token, public_key=public_key, jwt_algorithm=jwt_algorithm)



from setuptools import setup, find_packages

setup(
    name='token_sub_info',
    version='0.1.9',
    packages=find_packages(),
    install_requires=[
        'cryptography==3.4.8',
        'fastapi==0.104.1',
        'PyJWT==2.8.0'
    ],
    author='Ilya Mikhasik',
    author_email='imikhassik@gmail.com',
    description='A library for extracting information from jwt tokens.',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.12',
)



from uuid import UUID

from .utils import decode_token
from .exceptions.sub_exceptions.forbidden_exceptions import AllOrganizationAccessForbiddenException


def get_organizations(token: str, public_key: bytes, jwt_algorithm: str) -> list[UUID]:
    """
    Decode jwt token and return a list of organization UUIDs from token payload dictionary.

    :param token: jwt token to decode
    :type token: str
    :param public_key: public key used to decode the token
    :type public_key: bytes
    :param jwt_algorithm: jwt algorithm used to decode the token
    :type jwt_algorithm: str
    :returns: a list of organization UUIDs from token payload dictionary
    :rtype: list[UUID]
    """
    payload = decode_token(token, public_key, jwt_algorithm)
    organizations_ids: list[UUID] = payload.get("sub").get("organizations")
    if not organizations_ids:
        raise AllOrganizationAccessForbiddenException
    return organizations_ids



from ..main_exceptions import ForbiddenException


class AllOrganizationAccessForbiddenException(ForbiddenException):
    def __init__(self):
        super().__init__(code="FORBIDDEN", message="Пользователь не имеет доступа ни к одной из организаций.")
